@@ -2,7 +2,7 @@
 # housekeeping
 ########################################################################
 
-package LinkedList::Single v1.1.0;
+package LinkedList::Single v1.2.0;
 use v5.24;
 
 use Symbol;
@@ -107,62 +107,6 @@ my $verbose = $ENV{ VERBOSE } // '';
 # utility subs
 ########################################################################
 
-########################################################################
-# the issue here was fixed in later versions of Perl.
-# this can be removed to allow for automtic deletion.
-########################################################################
-#
-# perl's recursive cleanups croaks after 100 levels, kinda limits the
-# list size. fix is converting it to iterative by replacing the
-# head node.
-#
-# nasty business: simplest solution gets sig11's
-# in 5.8 & 5.10.1 with lists 2**15 long or more.
-# probelem is that destroy pukes after
-# returning. only fix so far is keeping the
-# heads alive permenantly (i.e., mem leak is
-# feature)
-#
-#    $head   = $head->[0]
-#    while $head->[0];
-#
-# fix is expanding the list in place, which
-# takes a bit more work.
-#
-#   @$head  = @{ $head->[0] }
-#   while $head->[0];
-#
-# weird thing is that it blows up after DESTROY returns,
-# not when the entry is deleted from $rootz{ $key }.
-#
-# net result: truncate works fine with the fast method,
-# DESTROY has to expand out the contents to make it work.
-#
-# note that doing this without the separate $node
-# variable gets the sigfault ( i.e.,
-#
-#   $head->[0] = @{ $head->[0] }
-#
-# blows up).
-#
-# see t/03*.t for example of testing this particular
-# issue.
-#
-#my $cleanup
-#= sub
-#{
-# no-op this for testing whether cleanup is still required.
-#
-#    my $node    = shift
-#    or return;
-#
-#    $node   = $node->[0]
-#    while $node->[0];
-#
-#    return
-#};
-########################################################################
-
 my $get_node_data
 = sub
 {
@@ -317,8 +261,21 @@ sub new
 
 sub clone
 {
-    state $pkg  = __PACKAGE__;
+    # clone a node with the same head as the current list.
 
+    my $listh   = shift;
+    my $clone   = $listh->new;
+
+    $$clone     = $$listh;
+
+    $rootz{ refaddr $clone }
+    = $rootz{ refaddr $listh };
+
+    $clone
+}
+
+sub clone_list
+{
     # clone the contents of $listh into a new list.
     #
     # reacall that the linked lists looks like a nested-
@@ -330,49 +287,22 @@ sub clone
     # by $clone.
 
     my $listh   = shift;
+    my $clone   = $listh->new;
 
-    my $clone
-    = do
-    {
-        if( @_ )
-        {
-            my $dest = shift
-            or croak "Bogus clone: false clone-into value";
-
-            blessed $dest
-            or croak "Bogus clone: '$dest' is not an object";
-
-            $dest->isa( __PACKAGE__ )
-            or croak "Bogus clone: '$dest' is not a '$pkg'";
-
-            $dest
-        }
-        else
-        {
-            $listh->new
-        }
-    };
-
-    # replacing both the root and current node 
-    # will release the linked list.
-
-    $$clone = $$listh;
-
-    $rootz{ refaddr $clone }
-    = $rootz{ refaddr $listh };
+    $clone->root_node->[0] = dclone $listh->root_node->[0];
 
     $clone
 }
 
 sub DESTROY
 {
-    my $head    = delete $rootz{ refaddr shift };
+    # note that this could be a clone whose 
+    # root entry was alrady zeroed by the 
+    # main list handler.
 
-    my $node    = $head->[0];
+    my $listh   = shift;
 
-#    $cleanup->( $node );
-
-    $#$head     = -1;
+    delete $rootz{ refaddr $listh };
 
     return
 }
@@ -389,8 +319,6 @@ sub truncate
 
     my $next    = splice @$node, 0, 1, [];
 
-#    $cleanup->( $next );
-
     $listh
 }
 
@@ -445,7 +373,7 @@ sub node
 sub set_meta
 {
     my $listh   = shift;
-    my $root    = $listh->root;
+    my $root    = $listh->root_node;
 
     @$root      = ( $root->[0], @_ );
 
@@ -486,7 +414,7 @@ sub has_nodes
 {
     # i.e., is the list populated?
 
-    my $root    = $_[0]->root;
+    my $root    = $_[0]->root_node;
 
     !! @{ $root->[0] }
 }
@@ -630,8 +558,6 @@ sub new_root
 
     my $key     = refaddr $listh;
 
-#    $cleanup->( delete $rootz{ $key } );
-
     $rootz{ $key }  = $root;
 
     $listh
@@ -645,8 +571,6 @@ sub new_head
 
     $head       = splice @$root, 0, 1, $head;
 
-#    $cleanup->( $head );
-
     $listh->head
 }
 
@@ -1219,9 +1143,10 @@ sub splice
     }
     else
     {
-#        $cleanup->( $dead );
+        # explicitly wipe the remainder by 
+        # zeroing out the next ref.
 
-        return
+        @$dead  = ();
     }
 }
 
@@ -1542,6 +1467,11 @@ the existing node's state has to be kept.
     # note that $listh is unaffected by the
     # head or walking via each.
 
+Note that this leaves two root nodes referencing the head
+node, with no way to determine which one to weaken. At
+that point cleaning up the clone is necessary for garbage
+collecting the list.
+
 =item clone_list
 
 This clones the entire list or a subset of it.
@@ -1553,16 +1483,11 @@ is assumed to be true and up to count nodes will be copied
 
     my $whole_clone = $listh->clone_list;
 
-    my $tail        = $listh->clone_list( tail  => 1    );
-
-    my $subset      = $listh->clone_list( count => $n   );
-
 Clones are relatively quick and can handle large subets of data
 if the nodes' data are small lists or references. This makes a
 reasonable way to analyze a subset of the list in a loop.
 
 
-
 =item set_meta add_meta get_meta
 
 These allow storing list-wide data in the head.
@@ -2033,9 +1958,9 @@ Steven Lembark <lembark@wrkhors.com>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2009-2011 Steven Lembark.
+Copyright (C) 2009-2021 Steven Lembark.
 
 =head1 LICENSE
 
-This code can be used under the same terms as v5.14 or
+This code can be used under the same terms as v5.28 or
 any later version of Perl.
